/*
	Librería para el control de un RTC
	Las funciones que permite esta librería son:
		-Establecer fecha y hora
		-Leer fecha y hora
		-Variar fecha y hora
		-Establecer alarmas
	Autor:		Pedro	Vera	Gallego
	última modificación 27/06/2017
*/
#include "RTC_lib.h"
#include <string.h>


/* Public */

Reloj::Reloj()
{
	
}	

/*
	RTC sirve para inicializar la clase y además inicia la comunicación I2C
*/

RTC::RTC ()
{
	Wire.begin();
}

/*
	La función leerHora se encarga de comunicarse con los registros 
	de datos del reloj, obteniendo el contenido de estos y añadiendolo
	en una variable x de tipo Reloj, previa decodificación
*/

Reloj RTC::leerHora()
{
	//Se llama a la función bit12_24 explicada en la linea 997
	bit12_24();
	//Se inicia la transmision I2C 
	Wire.beginTransmission(RTC_DIR);
	
    #if ARDUINO >= 100
        Wire.write(0);
    #else
        Wire.send(0);
    #endif
    Wire.endTransmission();
    Wire.requestFrom(RTC_DIR, 7);
	
	Reloj x;
	
	//Todas las lecturas de registro están en hexadecimal, por lo que se debe llamar a decodificar
    #if ARDUINO >= 100
	//Las lecturas van seguidas de una mascara AND para obviar los bits que no interesan
        x.seg    = decodificar(Wire.read() & 0b01111111);
        x.min    = decodificar(Wire.read() & 0b01111111);
		if(!AMPM)
		{
			//En caso de estar en modo 24h, se lee la hora y se almacena en x.hora
			x.hora    = decodificar(Wire.read() & 0b00111111);
			//Para tener tambien la hora en modo 12h, por si se quiere hacer un cambio de modo
			//se llama a la función de24a12 y se alamacena en x.hora12
			x.hora12  = de24a12(x.hora);
		}
		else
		{
			//Si el modo es 12h, lo contrario, se alamacena la leida en x.hora12
			x.hora12    = decodificar(Wire.read() & 0b00011111);
			//Se llama a de12a24 para almacenar el valor en x.hora
			x.hora		=  de12a24(x.hora12);
		}
        x.diaSem    = decodificar(Wire.read() & 0b00000111);
        x.dia        = decodificar(Wire.read() & 0b00111111);
        x.mes      = decodificar(Wire.read() & 0b00011111);
        x.anno       = decodificar(Wire.read());
	//Lo mismo, pero sustituyendo read por receive, en caso de ser otra versión de Arduino
    #else
        x.seg    = decodificar(Wire.receive() & 0b01111111);
        x.min    = decodificar(Wire.receive());
		if(!AMPM)
		{
			x.hora    = decodificar(Wire.receive() & 0b00111111);
			x.hora12  = de24a12(x.hora);
		}
		else
		{
			x.hora12    = decodificar(Wire.receive() & 0b00011111);
			x.hora		=  de12a24(x.hora12);
		}
        x.diaSem    = decodificar(Wire.receive() & 0b00000111);
        x.dia        = decodificar(Wire.receive());
        x.mes      = decodificar(Wire.receive());
        x.anno       = decodificar(Wire.receive());
	#endif
	
	//Esta función es de tipo Reloj, por lo que devuelve la estructura de los datos anteriores
	//estos datos se envian a compruebaCH, para comprobar si se debe producir un cambio horario
	return this->compruebaCH(x);
	
}

/*
	La función Modo12h, sirve para forzar al reloj a trabajar en modo 12 o 24h, para ello recibe
	un parámetro binario AM_PM, será ON=1 si se quiere modo 12h u OFF=0 si se quiere modo 24h
*/

void RTC::Modo12h(bool AM_PM)
{	
//Se hace una llamada a am_pm, ya que se necesita saber el modo actual, almacenado en la variable AMPM
	am_pm();
//Se alamcenan los valores actuales de los registros y se actualiza el modo AMPM
	Reloj x = this->leerHora();
	AMPM = AM_PM;
//La variable auxiliar AP sirve para que en la funcion escribeReloj no se vuelva a leerHora, ya que podría
//cambiar el valor de AM o PM, al valor anterior
	AP = true;	
//se pasan los valores actuales a escribeReloj, para que los introduzca en los registros
	this->escribeReloj(x);
}

/*
	escribeHora recibe los valores que se quieren forzar en los registros de hora, minutos y segundos
	son leidos y almacenados los valores actuales
	y se modifican los valores de hora, hora12, min y seg
*/

void RTC::escribeHora(uint8_t hora, uint8_t min, uint8_t seg)
{
	Reloj x = this->leerHora();
	AP = true;
	x.seg = seg;
	x.min = min;
	x.hora = hora;
	x.hora12 = de24a12(hora);
		
	this->escribeReloj(x);
}

/*
	escribeFecha recibe los valores que se quieren forzar en los registros de dia, mes y año
	como en el caso anterior, se leen los valores actuales y se modifican los nuevos
	el año se almacena desde la década
*/

void RTC::escribeFecha(uint8_t dia, uint8_t mes, uint16_t anno)
{
	Reloj x = this->leerHora();
	x.dia = dia;
	x.mes = mes;
	if(anno>99)
		x.anno = anno-2000;
	else
		x.anno = anno;
	//Para obtener el día de la semana se llama a la función calculaDiaSem, explicada abajo
	x.diaSem = calculaDiaSem(x.dia,x.mes,x.anno);
	this->escribeReloj(x);
}

/*
	escribeDiaSem sirve para forzar en el registro de dia de la semana el valor que se quiera
*/

void RTC::escribeDiaSem(uint8_t diaSem)
{
	Reloj x = this->leerHora();
	x.diaSem = diaSem;
	this->escribeReloj(x);
}

/*
	Las siguientes 6 funciones sirven para amumentar o decrementar en una unidad el valor
	de los registros y de los datos Reloj almacenados. Para ello reciben un parametro binario
	que será 1 si se quiere aumentar o 0 si se quiere decrementar
*/

void RTC::variaHora(bool varia)
{
	Reloj x = this->leerHora();
	if(varia)
	{
		++x.hora;
		++x.hora12;
		if(!AMPM && x.hora==24)
		x.hora=0; 
		else if(AMPM && x.hora12==13)
		{
			x.hora12=1;
			AMoPM = !AMoPM;
		}
	}
	else
	{
		--x.hora;
		--x.hora12;
		if(!AMPM && x.hora==-1)x.hora=23;
		else if(AMPM && x.hora12==-1)
		{
			x.hora12=12;
			AMoPM = !AMoPM;
		}
	}
	this->escribeReloj(x);
}

void RTC::variaMin(bool varia)
{
	Reloj x = this->leerHora();
	if(varia)
	{
		++x.min;
		if(x.min==60)x.min=0;
	}
	else
	{
		--x.min;
		if(x.min==-1)x.min=59;
	}
	x.seg = 0;
	this->escribeReloj(x);
}

void RTC::variaAnno(bool varia)
{
	Reloj x = this->leerHora();
	if(varia)
	{
		++x.anno;
		if(x.anno==100)x.anno=0;
	}
	else
	{
		--x.anno;
		if(x.anno==-1)x.anno=99;
	}
	x.diaSem = calculaDiaSem(x.dia,x.mes,x.anno);
	this->escribeReloj(x);
}

void RTC::variaMes(bool varia)
{
	Reloj x = this->leerHora();
	if(varia)
	{
		++x.mes;
		if(x.mes==13)x.mes=1;
	}
	else
	{
		--x.mes;
		if(x.mes==0)x.mes=12;
	}
	x.diaSem = calculaDiaSem(x.dia,x.mes,x.anno);
	this->escribeReloj(x);
}

void RTC::variaDia(bool varia)
{
	Reloj x = this->leerHora();
	if(varia)
	{
		++x.dia;
		if(x.mes==4||x.mes==6||x.mes==9||x.mes==11)
			{if(x.dia==31)x.dia=1;}
		else if(x.mes==2)
			{if(x.dia==30)x.dia=1;}
		
		else if(x.dia==32)x.dia=1;
	}
	else
	{
		--x.dia;
		if(x.mes==4||x.mes==6||x.mes==9||x.mes==11)
			{if(x.dia==0)x.dia=30;}
		else if(x.mes==2)
			{if(x.dia==0)x.dia=29;}
		else if(x.mes==1||x.mes==3||x.mes==5||x.mes==7||x.mes==8||x.mes==9)
			{if(x.dia==0)x.dia=31;}
	}
	x.diaSem = calculaDiaSem(x.dia,x.mes,x.anno);
	this->escribeReloj(x);
}

void RTC::variaDiaSem(bool varia)
{
	Reloj x = this->leerHora();
	if(varia)
	{
		++x.diaSem;
		if(x.diaSem==8)x.diaSem=1;
	}
	else
	{
		--x.diaSem;
		if(x.diaSem==0)x.diaSem=7;
	}
	this->escribeReloj(x);
}

/*
	verHora es una función que devuelve una cadena de caraceres, esta se genera
	con el valor de los registros hora, minutos y segundos
	El parámetro introducido, formato, puede ser FORMATO_LARGO o FORMATO_CORTO
	dependiendo si se quiere añadir a la cadena los segundos o no.
	Se llama a la función digitosDato, para ver si el dato tiene una o dos cifras
*/

String RTC::verHora(uint8_t formato)
{
	Reloj x;
	x=leerHora();
	String salida= String("");
	
if(!AMPM)
{
	if(digitosDato(x.hora))
		salida.concat("0");
	
		salida.concat(x.hora);
		salida.concat(":");
}
else
{
		if(digitosDato(x.hora12))
		salida.concat("0");
	
		salida.concat(x.hora12);
		salida.concat(":");
}

	if(digitosDato(x.min))
		salida.concat("0");
	
		salida.concat(x.min);
		
	//para el formato largo se incluyen los segundos
	if (formato==FORMATO_LARGO)
		{
		salida.concat(":");
		if(digitosDato(x.seg))
			salida.concat("0");
		salida.concat(x.seg);
		}
		
	//En caso de que sea AM/PM se añade a la cadena
	if(AMPM)
	{
		if(AMoPM)
			salida.concat(" PM");
		else
			salida.concat(" AM");
	}

	return salida;
}

/*
	verDia devuelve una cadena con el día de la semana.
	En este caso el FORMATO_CORTO devolveria solo las tres primeras letras
	y el FORMATO_LARGO devolvería el día completo
*/

String RTC::verDia(uint8_t formato)
{	
	Reloj x;
	x=leerHora();
	String salida=String("");
	
	switch (x.diaSem)
	{
		case LUNES:
			salida="Lunes";
			break;
		case MARTES:
			salida="Martes";
			break;
		case MIERCOLES:
			salida="Miercoles";
			break;
		case JUEVES:
			salida="Jueves";
			break;
		case VIERNES:
			salida="Viernes";
			break;
		case SABADO:
			salida="Sabado";
			break;
		case DOMINGO:
			salida="Domingo";
			break;
	}
	//En caso del formato corto, se muestra solo las tres primeras letras, como "Lun"
	if (formato==FORMATO_CORTO)
		salida[3]=0;
	return salida;
}

/*
	verFecha devuelve una cadena con el día, mes y año.
	El parametro formato sirve para poner el año completo o solo la década
	y el orden para cambiar el formato de la cadena:
	1->DD/MM/AA		2->AA/MM/DD		3->MM/DD/AA
*/

String RTC::verFecha(uint8_t formato, uint8_t orden)
{
	uint16_t an=0;
	Reloj x;
	x=leerHora();
	String salida=String("");
	
	if (digitosDato(x.dia))
		salida.concat("0");
	
		salida.concat(x.dia);
		salida.concat("/");
	
	if (digitosDato(x.mes))
		salida.concat("0");	

		salida.concat(x.mes);
		salida.concat("/");

	if (formato==FORMATO_CORTO)
		{
			if (digitosDato(x.anno))
				salida.concat("0");
			
				salida.concat(x.anno);
		}
	else
		{
		an=	x.anno+2000;
		salida.concat(an);	
		}	
	switch (orden)
	{
		case 1:
		//No se hace nada, la cadena queda tal cual-> DD/MM/AA o DD/MM/AAAA
			break;
		case 2:
		if(formato==FORMATO_LARGO)
		{
		//Ordenamos la cadena dándo la vuelta y cambiando posiciones -> AAAA/MM/DD
			salida=invierte(salida);
			salida=cambio(0,3,salida);
			salida=cambio(1,2,salida);
			salida=cambio(5,6,salida);
			salida=cambio(8,9,salida);
		}
		else
		{
		//Intercambiamos los días por lo años-> AA/MM/DD
			salida=cambio(0,6,salida);
			salida=cambio(1,7,salida);
		}
			break;

		case 3:
		//Ordenamos la cadena para ser MM/DD/AAAA o MM/DD/AA
			salida=cambio(0,3,salida);
			salida=cambio(1,4,salida);
			break;
	}
	
	return salida;
}

/*
	verMes devuelve una cadena con el nombre del mes, siendo el formato como en el caso
	del dia de la semana, las tres primeras letras el corto y completo el largo
*/

String RTC::verMes(uint8_t formato)
{
	Reloj x;
	x=leerHora();
	String salida=String("");
	
	switch (x.mes)
	{
		case 1:
			salida="Enero";
			break;
		case 2:
			salida="Febrero";
			break;
		case 3:
			salida="Marzo";
			break;
		case 4:
			salida="Abril";
			break;
		case 5:
			salida="Mayo";
			break;
		case 6:
			salida="Junio";
			break;
		case 7:
			salida="Julio";
			break;
		case 8:
			salida="Agosto";
			break;
		case 9:
			salida="Septiembre";
			break;
		case 10:
			salida="Octubre";
			break;
		case 11:
			salida="Noviembre";
			break;
		case 12:
			salida="Diciembre";
			break;
	}
	//Si elegimos formato corto eliminamos a partir de la posicion 3
	if (formato==FORMATO_CORTO)
		salida[3]=0;
	return salida;
}

/*
	verTemperatura es una función que devuelve el valor del registro temperatura
	
*/

float RTC::verTemperatura()
{
	byte temp=0;
	//Lectura del registro
	Wire.beginTransmission(RTC_DIR);
	#if ARDUINO >= 100
		Wire.write(uint8_t(TEMP_REG1));
	#else
		Wire.send(uint8_t(TEMP_REG1));
	#endif
	Wire.endTransmission();
	Wire.requestFrom(RTC_DIR,2);
	
	#if ARDUINO>=100
		temp=Wire.read();
	#else
		temp=Wire.receive();
	#endif
	//Se pasa la lectura del registro a ºC
	temp = float(temp) + 0.25*(Wire.read()>>6);
	return temp;
}

//Función para solicitar la lectura de un solo registro

uint8_t RTC::leeRegistro(uint8_t direccion)
{
	uint8_t reg;
	
	Wire.beginTransmission(RTC_DIR);
    Wire.write(direccion);
    Wire.endTransmission();
    Wire.requestFrom(RTC_DIR, 1);
    reg = Wire.read();
	
	return reg;
}

/*
	La función escribeRegistro es para escribir en un registro escpecífico
	se le pasa la dirección del registroa escribir y el valor a escribir
*/

void RTC::escribeRegistro(uint8_t direccion,uint8_t valor)
{
	Wire.beginTransmission(RTC_DIR);
    Wire.write(direccion);
    Wire.write(valor);
    Wire.endTransmission();
}

/*
	alarmasON es una función que habilita las interrupciones, escribiendo en el registro de control la máscara
	que pone a 1 RS1, RS2, INT, AE1 y AE2
	además determina el tipo de alarma, escribiendo la máscara correspondiente a cada tipo, en la alarma1 o la 2
*/

void RTC::alarmasON(uint8_t tipo)
{
	//ALARMA 1 ==> //Tipo 1: Cada Segundo //Tipo 2: Coincidencia de segundos //Tipo 3: Coincidencia de min/seg //Tipo 4: Coincidencia de hora/min/seg
	//ALARMA 2 ==> //Tipo 5: Cada minuto //Tipo 6: Coincidencia de hora/min //Tipo 7: Coincidencia de diaSem/hora/min  //Tipo 8: Coincidencia de dia/hora/min 
	if(tipo>8||tipo<1) tipo=8;
	
	tipoAl = tipo;
	
	//Los dos bits menos significativos son los que habilitan las alarmas 1 y 2, el tercero habilita las interrupciones
	uint8_t regControl=0;
		regControl |= 0b00011111; 
	escribeRegistro(CTRL_REG, regControl);
	
	switch(tipo)
	{
		//Establecer la mascara para cada alarma
		case 1:
			escribeRegistro(AL1SEG_REG, 0b10000000);
			escribeRegistro(AL1MIN_REG, 0b10000000);
			escribeRegistro(AL1HORA_REG, 0b10000000);
			escribeRegistro(AL1DIA_REG, 0b10000000);
			break;
		
		case 2:
			escribeRegistro(AL1SEG_REG, 0b00000000);
			escribeRegistro(AL1MIN_REG, 0b10000000);
			escribeRegistro(AL1HORA_REG, 0b10000000);
			escribeRegistro(AL1DIA_REG, 0b10000000);
			break;
			
		case 3:
			escribeRegistro(AL1SEG_REG, 0b00000000);
			escribeRegistro(AL1MIN_REG, 0b00000000);
			escribeRegistro(AL1HORA_REG, 0b10000000);
			escribeRegistro(AL1DIA_REG, 0b10000000);
			break;
			
		case 4:
			escribeRegistro(AL1SEG_REG, 0b00000000);
			escribeRegistro(AL1MIN_REG, 0b00000000);
			escribeRegistro(AL1HORA_REG, 0b00000000);
			escribeRegistro(AL1DIA_REG, 0b10000000);
			break;
			
		case 5:
	
			escribeRegistro(AL2MIN_REG, 0b10000000);
			escribeRegistro(AL2HORA_REG, 0b10000000);
			escribeRegistro(AL2DIA_REG, 0b10000000);
			break;
			
		case 6:
	
			escribeRegistro(AL2MIN_REG, 0b00000000);
			escribeRegistro(AL2HORA_REG, 0b00000000);
			escribeRegistro(AL2DIA_REG, 0b10000000);
			break;		
			
		case 7:
	
			escribeRegistro(AL2MIN_REG, 0b00000000);
			escribeRegistro(AL2HORA_REG, 0b00000000);
			escribeRegistro(AL2DIA_REG, 0b01000000);
			break;

		case 8:
	
			escribeRegistro(AL2MIN_REG, 0b00000000);
			escribeRegistro(AL2HORA_REG, 0b00000000);
			escribeRegistro(AL2DIA_REG, 0b00000000);
			break;			
	}
}

/*
	escribeAlarma es una función que recibe unos parametros, de día, hora, minutos y segundos
	para escribirlos en los registros de la alarma 1 o la alarma 2, dependiendo el tipo, 
	y así al coincidir los registros de hora con los de alarma, activar en el registro de STATUS_REG
	el bit correspondiente al flag de la alarma 1 o alarma 2
*/ 

void RTC::escribeAlarma(uint8_t dia, uint8_t hora, uint8_t min, uint8_t seg)
{
	//Comprobar si es modo 12 o 24h
	am_pm();
	
	//Variable que almacenará el contenido del registro actualmente
	uint8_t reg=0;
	
	//Condicional para los cuatro primeros tipos de alarma, que no afectan a los días
	if(tipoAl==1 || tipoAl==2 || tipoAl==3 || tipoAl==4)
	{
		//Se almacena el valor en reg y se le aplica una máscara AND, para que no quede ningún bit que pueda cambiar el valor nuevo
		reg = leeRegistro(AL1SEG_REG);
			escribeRegistro(AL1SEG_REG, reg &10000000 | codificar(seg));
		reg = leeRegistro(AL1MIN_REG);
			escribeRegistro(AL1MIN_REG, reg &10000000 |  codificar(min));
		reg = leeRegistro(AL1HORA_REG);
		if(!AMPM)
			escribeRegistro(AL1HORA_REG, reg &10000000 | codificar(hora));
		else
		{
			//En caso de ser modo 12h se pone a 1 el bit 6 y dependiendo de AM o PM el 5
			hora = de24a12(hora);
			if(AMoPM==AM)
			escribeRegistro(AL1HORA_REG, reg | 0b01000000 | codificar(hora));
			else
			escribeRegistro(AL1HORA_REG, reg | 0b01100000 | codificar(hora));
		}
	}
	//El resto de alarmas afectan a los días
	else	
	{
		reg = leeRegistro(AL2MIN_REG);
			escribeRegistro(AL2MIN_REG, reg &10000000 | codificar(min));
		reg = leeRegistro(AL2HORA_REG);	
		if(!AMPM)
			escribeRegistro(AL2HORA_REG, reg &10000000 | codificar(hora));
		else
		{
			hora = de24a12(hora);
			if(AMoPM==AM)
			escribeRegistro(AL2HORA_REG, reg &10000000 | 0b01000000 | codificar(hora));
			else
			escribeRegistro(AL2HORA_REG, reg &10000000 | 0b01100000 | codificar(hora));
		}	
		
		reg = leeRegistro(AL2DIA_REG);	
			escribeRegistro(AL2DIA_REG, reg &10000000 |  codificar(dia));
	}
	
	
	
}

/*
	limpiaAlarma se utilizará para limpiar los flags del registro STATUS_REG cuando
	se haya producido una alarma
*/

void RTC::limpiaAlarma()
{
	//Se limpian los flags de ambas alarmas A1F y A2F
    uint8_t regStat = leeRegistro(STATUS_REG);
	//La variable alarm, selecciona la alarma a borrar
		regStat &= 0b11111100;
	
    escribeRegistro(STATUS_REG, regStat);
}

// privadas

/*
	Las siguientes 7 funciones sirven para comprobar que los parametros a escribir en los registros
	están entre los valores normales de hora, minutos, segundos, dia, mes ,dia de la semana y año
*/

bool RTC::horaEntre(uint8_t hora)
{
	if((hora>=0) && (hora<=23)) return true;
	else return false;
}

bool RTC::hora12Entre(uint8_t hora12)
{
	if((hora12>=1) && (hora12<=12)) return true;
	else return false;
}

bool RTC::minsegEntre(uint8_t minseg)
{
	if((minseg>=0) && (minseg<=59)) return true;
	else return false;
}

bool RTC::diaEntre(uint8_t dia, uint8_t mes)
{
	uint8_t diamax=0;
	
	if(mes==4||mes==6||mes==9||mes==11)
		diamax=30;
	else if(mes==2)
			diamax=29;
		else
			diamax=31;
	if((dia>=1) && (dia<=diamax)) return true;
	else return false;
		
}
bool RTC::mesEntre(uint8_t mes)
{
	if((mes>=1) && (mes<=12)) return true;
	else return false;
}
bool RTC::annoEntre(uint8_t anno)
{
	if(anno>=0 && anno<=99) return true;
	else return false;
}
bool RTC::diaSemEntre(uint8_t diaSem)
{
	if((diaSem>0) && (diaSem<=7)) return true;
	else return false;
}

/*
	La función cambio se utiliza para ordenar la cadena fecha, y es llamada por verFecha
	los parámetros que necesita son la posición en la cadena de los dos caracteres a intercambiar
	y la cadena entera
*/

String RTC::cambio(int pos1, int pos2, String salida )
{
	String aux = salida;
	aux[pos1]=salida[pos2];
	aux[pos2]=salida[pos1];
	return aux;
}

/*
	invierte, como la anterior es llamada por verFecha, para hacer más rápido el cambio en caso de 
	estar en formato DD/MM/AAAA. Solo necesita que se le pase la cadena.
*/

String RTC::invierte(String salida )
{
	String aux = salida;

	for(int i=0; i<5;++i)
	{
		aux[i]=salida[9-i];
	}
	for(int i=5; i<10;++i)
	{
		aux[i]=salida[9-i];
	}
	return aux;
}

/*
	La función digitosDato, se llama en las funciones de verX, para comprobar si el dato es de
	una cifra o de dos. Se le pasa el dato y devuelve un 1 lógico si es de una cifra o un 0 si es de dos
*/

bool RTC::digitosDato(int dato)
{
	if (dato<10)
		return true;
	else
		return false;
}

/*
	escribeReloj es la función que se encarga de actualizar los valores de los registros del RTC
	se le debe pasar la variable x con los valores a escribir. A la hora de escribir cada dato,
	se comprueba con xEntre, si está entre unos valores correctos y con la función codificar se pasa
	a hexadecimal para escribirla en el registro mediante wire.write o wire.send, dependiendo la versión
*/

void RTC::escribeReloj( Reloj x)
{
	//Se leen los valores actuales del reloj, para que en caso de que los nuevos no sean válidos
	//se reescriban los antiguos
	Reloj actual;
	if(!AP)
		actual = this->leerHora();
	
	AP = false;
	
	Wire.beginTransmission(RTC_DIR);
	
	
	#if ARDUINO >= 100
		Wire.write(0);
		
		if(minsegEntre(x.seg))
			Wire.write(codificar(x.seg));
		else
			Wire.write(codificar(actual.seg));
		
		if(minsegEntre(x.min))
			Wire.write(codificar(x.min));
		else
			Wire.write(codificar(actual.min));
		
	if(!AMPM)
	{
		if(horaEntre(x.hora))
			Wire.write(codificar(x.hora) & 0b00111111);
		else
			Wire.write(codificar(actual.hora));
	}
	else
	{
		if(hora12Entre(x.hora12))
		{
			if(AMoPM==PM)
				Wire.write((codificar(x.hora12)& 0b00011111) | 0b01100000);
			else
				Wire.write((codificar(x.hora12)& 0b00011111) | 0b01000000);
		}
		else
			Wire.write(codificar(actual.hora12));	
	}
	
		if(diaSemEntre(x.diaSem))
			Wire.write(codificar(x.diaSem));
		else
			Wire.write(codificar(actual.diaSem));
		
		if(diaEntre(x.dia,x.mes))
			Wire.write(codificar(x.dia));
		else
			Wire.write(codificar(actual.dia));
		
		if(mesEntre(x.mes))
			Wire.write(codificar(x.mes));
		else
			Wire.write(codificar(actual.mes));
		
		if(annoEntre(x.anno))
			Wire.write(codificar(x.anno));
		else
			Wire.write(codificar(actual.anno));

	#else
		Wire.send(0);
		
		if(minsegEntre(x.seg))
			Wire.send(codificar(x.seg));
		else
			Wire.send(codificar(actual.seg));
		
		if(minsegEntre(x.min))
			Wire.send(codificar(x.min));
		else
			Wire.send(codificar(actual.min));
		
	if(!AMPM)
	{
		if(horaEntre(x.hora))
			Wire.send(codificar(x.hora) & 0b00111111);
		else
			Wire.send(codificar(actual.hora));
	}
	else
	{
		if(hora12Entre(x.hora12))
		{
			if(AMoPM==PM)
				Wire.send((codificar(x.hora12)& 0b00011111) | 0b01100000);
			else
				Wire.send((codificar(x.hora12)& 0b00011111) | 0b01000000);
		}
		else
			Wire.send(codificar(actual.hora12));	
	}
	
		if(diaSemEntre(x.diaSem))
			Wire.send(codificar(x.diaSem));
		else
			Wire.send(codificar(actual.diaSem));
		
		if(diaEntre(x.dia))
			Wire.send(codificar(x.dia));
		else
			Wire.send(codificar(actual.dia));
		
		if(mesEntre(x.mes))
			Wire.send(codificar(x.mes));
		else
			Wire.send(codificar(actual.mes));
		
		if(annoEntre(x.anno))
			Wire.send(codificar(x.anno));
		else
			Wire.send(codificar(actual.anno));

	#endif
	Wire.endTransmission();	
}

//Pasar de hexadecimal a decimal

uint8_t	RTC::decodificar(uint8_t valor)
{
	uint8_t decodificado = ((valor / 16) * 10) + (valor % 16);
	
	return decodificado;
}

//Pasar de decimal a hexadecimal

uint8_t RTC::codificar(uint8_t valor)
{
	uint8_t codificado = ((valor / 10) * 16) + (valor % 10);
	
	return codificado;
}


/*
	La función am_pm comprueba si el bit del registro hora está a 1 (modo 12h)
	o a 0 (modo 24h) y varía la variable AMPM
*/

void RTC::am_pm()
{
	if((leeRegistro(HORA_REG)&0b01000000)==0b01000000)
		AMPM = ON;
	else
		AMPM = OFF;
}

/*
	La función bit12_24 se asegura de poner el bit de 12h o 24h a 1 o 0 respectivamente
	y en caso de ser modo 12h, comprueba si el bit 5 es 1 o 0, para saber si es PM o AM/PM
	respectivamente
*/

void RTC::bit12_24()
{
	uint8_t hReg = leeRegistro(HORA_REG);

	if(AMPM)
	{
		escribeRegistro(HORA_REG, ((hReg & 0b00111111) |0b01000000));
		if((hReg & 0b00100000)==0b00100000)
			AMoPM = PM;
		else
			AMoPM = AM;	
	}
	else
		escribeRegistro(HORA_REG, hReg & 0b00111111);
}

/*
	La función de12a24 simplemente recibe la hora en modo 12h y la devuelve en modo 24h
*/

uint8_t RTC::de12a24(uint8_t hora)
{
	uint8_t hora24 = hora;
		
	if(AMoPM == PM)
	{
		if(hora < 12)
			hora24 +=12;
	}
	else
		if(hora == 12) 
			hora24 = 0;	
	return hora24;
}

/*
	La función de24a12 recibe la hora en modo 24h y la devuelve en modo 12h
*/

uint8_t RTC::de24a12(uint8_t hora)
{
	uint8_t hora12 = hora;
	if(hora>=12)
	{
		if(hora>12)
			hora12 -=12;
		AMoPM=PM;
	}
	else
		AMoPM=AM;
	if(hora==0) 
		hora12 = 12;
	return hora12;
}

/*
	compruebaCH recibe la lectura de los registros en la variable x 
	y viendo si es el último domingo de Marzo o de Octubre, cambia el valor de horaMas y horaMenos
*/

Reloj RTC::compruebaCH(Reloj x)
{
		uint8_t diaSemana=0;
		
	//Si horaMas u horaMenos es 0, se han inicializado, por lo que se calcula
	//el dia de ambos cambios horarios 
	
        if( horaMas == 0 || horaMenos == 0 )
        {
		//El día de adelantar hora es el último domingo de marzo, si ya pasó marzo se calcula el del siguiente año
			diaSemana = calculaDiaSem( 31, 3, (x.mes <= 3 ? x.anno : x.anno+1));
		//La siguiente condición es debida a que calculaDiaSem devuelve un 7 si es domingo
			if (diaSemana == 7) diaSemana = 0;
            horaMas  = 31 - diaSemana;
			diaSemana=0;
		//El día de atrasar hora es el último domingo de octubre, si ya pasó octubre se calcula el del siguiente año
			diaSemana = calculaDiaSem( 31, 10, (x.mes <= 10 ? x.anno : x.anno+1));
			if (diaSemana == 7) diaSemana = 0;
            horaMenos = 31 - diaSemana;
		//Si la fecha no está tras el cambio de horaMas y  antes de horaMenos, cH será FALSE
            cH  = ((x.mes == 3 && x.dia >= horaMas && x.hora > 2) || x.mes==4 || x.mes==5 || x.mes==6 || x.mes==7 || x.mes==8 || x.mes==9 || (x.mes == 10 && x.dia <= horaMenos && x.hora < 3)) ? true : false;
        }
	//Si cH es 0 y es Marzo coincidiendo con el último domingo del mes (horaMas), cuando sean las 2 se suma una hora y cH pasa a 1
        if( !cH  &&  x.mes == 3  &&  x.dia == horaMas  &&  x.hora == 2 )
        {
            x.hora++;
            cH = true;
		//Se calcula el proximo cambio de hora
			diaSemana = calculaDiaSem( 31, 10, x.anno+1);
			if (diaSemana == 7) diaSemana = 0;
            horaMas  = 31 - diaSemana;
            this->escribeReloj(x);
        }
        else if( cH  &&  x.mes == 10  &&  x.dia == horaMenos  &&  x.hora == 3 )
        {
            x.hora--;
            cH = false;
			diaSemana = calculaDiaSem( 31, 10, x.anno+1);
			if (diaSemana == 7) diaSemana = 0;
            horaMenos = 31 - diaSemana;
            this->escribeReloj(x);
        }

    x.cambioHorario = cH;
    return x;
}

/*
	La función calculaDiaSem recibe los valores de la fecha y mediante el algoritmo de Zeller
	calcula el día de la semana que es, siendo 0 el Domingo y 1-6 del Lunes al Sábado 
*/

uint8_t RTC::calculaDiaSem( uint8_t dia, uint8_t mes, uint16_t anno )
{
	uint8_t diaS=0;
    anno+=2000;
	if( mes > 2 )
        mes -= 2;
    else
    {
        mes += 10;
        anno -= 1;
    }
    int a = anno % 100;
    int b = anno / 100;
    diaS = (700 + ((26 * mes - 2) / 10) + dia + a + a / 4 + b / 4 - 2 * b) % 7;
	if(diaS==0)
		diaS=7;
	return diaS;
	}

