/*
  RTC_lib.h - Librería de Arduino para el control de los dispositivos DS1307 al DS3231, los cuales son relojes
  de tiempo real de la marca DALLAS.
  
  Autor:	Pedro	Vera	Gallego
  última modificación:	27/07/2017

*/
#ifndef RTC_lib_h
#define RTC_lib_h

//Dependiendo de la version de Ardiomp se añade una cabecera
#if defined(ARDUINO) && ARDUINO >= 100
	#include "Arduino.h"
#else
	#include "WProgram.h"
#endif

#include "Wire.h"

#define FORMATO_LARGO	1
#define FORMATO_CORTO	2

#define		ON		1
#define		OFF		0

#define		PM		1
#define		AM		0

//Definición de dias de la semana
#define		LUNES		1
#define		MARTES		2
#define		MIERCOLES	3
#define		JUEVES		4
#define		VIERNES		5
#define		SABADO		6
#define		DOMINGO		7

//Direccion de los registros de tiempo, para comunicación I2C
#define RTC_DIR 0x68

//Registros del dispositivo
#define SEG_REG			0x00  
#define MIN_REG			0x01  
#define HORA_REG		0x02
#define DSEM_REG		0x03
#define DIA_REG			0x04
#define MES_REG			0x05
#define ANNO_REG		0x06

#define AL1SEG_REG		0x07
#define AL1MIN_REG		0x08
#define AL1HORA_REG		0x09
#define AL1DIA_REG		0x0A

#define AL2MIN_REG		0x0B
#define AL2HORA_REG		0x0C
#define AL2DIA_REG		0x0D

#define CTRL_REG		0x0E
#define STATUS_REG      0x0F
#define OFFSET_REG		0x10
#define TEMP_REG1		0x11
#define TEMP_REG2		0x12

//Clase Reloj, que incluirá las variables necesarias para el control del reloj
// horas, minutos, segundos, fecha y dia de la semana

class Reloj
{
	public:

		uint8_t		dia;
		uint8_t		mes;
		uint16_t	anno;
		uint8_t		diaSem;
		uint8_t		hora;
		uint8_t		hora12;
		uint8_t		min;
		uint8_t		seg;
		bool		cambioHorario;

	Reloj();
};


//Clase RTC, que contiene las funciones necesarias para el control del reloj

class RTC
{
public:

//Función para inicializar el RTC y el wire.begin()
	RTC();
	
//Función que lee el contenido de los registros y los almacena
	Reloj	leerHora();
	
//Función para cambiar de modo horario, 12 AM/PM o 24h
	void Modo12h(bool AM_PM);
	
	//Funciones para forzar una fecha o una hora
	void	escribeHora(uint8_t hora=0, uint8_t min=0, uint8_t seg=0);
	void	escribeFecha(uint8_t dia, uint8_t mes, uint16_t anno);
	void	escribeDiaSem(uint8_t diaSem=1);
	
	//Funciones para incrementar o decrementar en una unidad la fecha, hora o dia de la semana
	void	variaHora(bool varia);
	void	variaMin(bool varia);
	void	variaAnno(bool varia);
	void	variaMes(bool varia);
	void	variaDia(bool varia);
	void	variaDiaSem(bool varia);
	
	//Funciones para pasar el valor de los datos a una cadena de caracteres
	//para mostrarlos en un formato entendible
	String	verHora(uint8_t formato=FORMATO_LARGO);
	String	verFecha(uint8_t formato=FORMATO_LARGO, uint8_t orden=1); //El formato y orden se inicializa a DD/MM/AAAA, pudiendo ser AAAA/MM/DD o MM/DD/AAAA
	String	verDia(uint8_t formato=FORMATO_LARGO);
	String	verMes(uint8_t formato=FORMATO_LARGO);
	
	//El DS3231 tiene un sensor de temperatura
	float	verTemperatura();
	
	//Funciones para leer y escribir un registro por separado
	uint8_t leeRegistro(uint8_t direccion);
	void escribeRegistro(uint8_t direccion,uint8_t valor);

	//Interrupciones/Alarmas, por defecto será una alarma cuando coincida dia/hora/min
	void alarmasON(uint8_t tipo=8);
	//Función para establecer la alarma
	void escribeAlarma(uint8_t dia=1, uint8_t hora=0, uint8_t min=0, uint8_t seg=0);
	//Función que limpia el flag de las alarmas que ya han provocado una interrupcion
	void limpiaAlarma();
	
private:

	//Funciones para comprobar si el valor introducido esta entre los margenes de los valores minimos y maximos
	bool horaEntre(uint8_t hora);
	bool hora12Entre(uint8_t hora12);
	bool minsegEntre(uint8_t minseg);
	bool diaEntre(uint8_t dia,uint8_t mes);
	bool mesEntre(uint8_t mes);
	bool annoEntre(uint8_t anno);
	bool diaSemEntre(uint8_t diaSem);
	
	//Funciones que cambian el orden de la cadena fecha, para la función verFecha
	String	cambio(int pos1, int pos2, String salida);
	String	invierte(String salida);
	
	//Función para comprobar si el dato es de un dígito o dos
	//al añadirlos a la cadena en verHora, verFecha o verDia
	bool digitosDato(int dato);

	//Decodificar pasa de hexadecimal a decimal
	uint8_t	decodificar(uint8_t valor);
	//Codificar pasa de decimal a hexadecimal
	uint8_t	codificar(uint8_t valor);
	
	
	//Función para leer el estado actual de AMPM (modo 12 o 24h)
	void am_pm();
	
	//Asegura que el bit de 12 o 24h está correcto y determina AM o PM
	void bit12_24();
	
	//Funciones para cambiar de formato la hora
	uint8_t de24a12(uint8_t hora);
	uint8_t de12a24(uint8_t hora);
	
	//Función para escribir datos en los registros
	//a esta función es a la que llaman todas las de escribir
	//datos así como las de variarlos
	void escribeReloj(Reloj datos);
	
	//Variables para indicar si hay cambio horario
	bool cH=false;
	uint8_t horaMas=0;
    uint8_t horaMenos=0;
	
	//Variables para el cambio de modo, 12 o 24h
	bool AMPM = false;
	bool AMoPM = AM;
	bool AP = false;
	
	//Variable que indica el tipo de alarma
	uint8_t tipoAl=8;
	
	//Función que comprueba si se produce cambio de hora
	Reloj compruebaCH(Reloj datos);
	
	//Función que aplica el algoritmo de Zeller para calcular el dia de la semana
	uint8_t calculaDiaSem( uint8_t dia, uint8_t mes, uint16_t anno );
};

#endif
